#include <iostream>

#include "myinterpreter.h"
#include "myapplication.h"

#include "octave/parse.h"

#include <QDebug>

// in libgui/graphics/__init_qt__.cc
// Linker errors undefined if try and call this though. Can I
// link to __init_qt__.oct ??
extern octave_value_list F__init_qt__ (octave::interpreter& interp,
                                       const octave_value_list& args,
                                       int nargout);

MyInterpreter::MyInterpreter(MyApplication *parent)
    : QObject(parent)
    , octave::interpreter(parent)
{
    // Mostly taken from https://hg.savannah.gnu.org/hgweb/octave/file/0089f0213384/examples/code/embedded.cc

    qDebug() << "Initialising MyInterpreter";
    try
    {
        initialize_history(false);
        initialize_load_path();
        // Add our custom .m files to the path
        get_load_path().prepend("octave-src/", true);
        initialize();
        int status = -1;

        if (! initialized ())
        {
            qDebug() << "Octave interpreter initialization failed!";
            exit (status);
        }
        else
        {
            qDebug() << "Octave interpreter initialised okay";
        }
    }
    catch (const octave::exit_exception& ex)
    {
        qDebug() << "Octave interpreter exited with status = "
                 << ex.exit_status ();
    }
    catch (const octave::execution_exception&)
    {
        qDebug() << "error encountered in Octave evaluator!";
    }

}

// Try and register the Qt Graphics stack!
void MyInterpreter::registerGraphics(void)
{
    octave_value_list out;
    try
    {
        // Try calling __init_qt__ through the interpreter
        int flag = 1;
        out = this->eval_string(std::string("__init_qt__"), false, flag);
        qDebug() << "__init_qt__ now finished. Have "
                 << out.length() << " output args.";

        // What's the output from eval_string??
        qDebug() << "Class: "   << out(0).class_name().c_str() << ". "
                 << "Type: "    << out(0).type_name().c_str() << ". "
                 << "Type ID: " << out(0).type_id();

        // Try and get some info on whatever has been returned?
        out(0).print_info(std::cerr);

        // Something I tried doing, didn't get very far with this...
        // m_interpreter->get_gtk_manager();
        //F__init_qt__(*m_interpreter, args, 0);
        //octave::symbol_table &symtab = m_interpreter->get_symbol_table();

        qDebug();
        qDebug() << "Run 'graphics_toolkit qt' in the Octave Interpreter.";

        // Q. Can we select the qt graphics toolkit this way?
        // A. No. This throws an exception.
        out = this->eval_string(std::string("graphics_toolkit qt"), false, flag);
        qDebug() << "graphics_toolkit qt now finished. Have "
                 << out.length() << " output args.";

        // Try and get some info on whatever has been returned?
        out(0).print_info(std::cerr);
    }
    catch (const octave::execution_exception& ex)
    {
        qDebug() << "error encountered in Octave evaluator!";
        qDebug() << ex.info().c_str();
        return;
    }

    return;

    /*
    // Try and register qt by creating a base_graphics_toolkit.
    // Almost definitely not the way to go...

    std::cerr << "Registering Qt Graphics" << std::endl;
    base_graphics_toolkit base_qttk = base_graphics_toolkit(std::string("qt"));
    graphics_toolkit qttk = graphics_toolkit(&base_qttk);

    m_interpreter->get_gtk_manager().load_toolkit(qttk);
    m_interpreter->get_gtk_manager().register_toolkit("qt");
    std::cerr << "Graphics registered" << std::endl;
    */
}

// Print the current graphics toolkits available loaded in this interpreter's
// gtk_manager.
void MyInterpreter::printGraphics(void)
{
    Cell available_toolkits = this->get_gtk_manager().available_toolkits_list();
    octave_value toolkit;
    qDebug() << "Have " << available_toolkits.numel() << " toolkits available.";
    for (octave_idx_type i = 0; i < available_toolkits.numel() ; i++ ) {
        toolkit = available_toolkits(i);
        qDebug() << "Available Toolkit: "
                  << available_toolkits(i).string_value().c_str();
    }

    // Check current graphics_toolkit registered in the interpreter.
    int flag = 1;
    qDebug() << "Current graphics toolkit:-";
    this->eval_string("graphics_toolkit", false, flag);
}

// Run a simple .m file
void MyInterpreter::runTestFigure(octave_value_list *args)
{
    qDebug() << "Calling testFigure.m";
    octave::feval("testFigure", *args, 0);
    qDebug() << "testFigure.m complete.";
}
